#!/bin/sh

#apt-get install mtd-utils qemu


FIRMWARE=http://www.eq-3.de/Downloads/Software/HM-CCU2-Firmware_Updates/HM-CCU2-2.15.5/HM-CCU-2.15.5.tar.gz
TEMP=/tmp/rfd
INSTALL_DIR=/opt/rfd

# Prepare
fnc_prepare() {
	umount $TEMP/ubifs
	modprobe -r ubifs ubi nandsim
	rm -r $TEMP
	mkdir -p $TEMP/ubifs
	mkdir -p $TEMP/rfd
	mkdir -p $TEMP/download
	mkdir -p $TEMP/rfd/etc/config/firmware
	mkdir $TEMP/rfd/lib
	mkdir $TEMP/rfd/bin
	mkdir -p $TEMP/rfd/var/log
}

# Download Firmware
fnc_download() {
	cd $TEMP/download
	wget -O firmware.tar.gz $FIRMWARE
	tar -xvzf firmware.tar.gz
}

# Mount CCU Firmware FS
fnc_mount_firmware() {
	modprobe nandsim first_id_byte=0x20 second_id_byte=0xaa third_id_byte=0x00 fourth_id_byte=0x15
	ubiformat /dev/mtd0 -f $TEMP/download/rootfs.ubi -O 2048
	modprobe ubi
	ubiattach /dev/ubi_ctrl -m 0 -O 2048
	mount -t ubifs ubi0:root $TEMP/ubifs

}

# Copy files
fnc_copy_files() {
	qemu-arm -L $TEMP/ubifs $TEMP/ubifs/lib/ld-linux.so.3 --list $TEMP/ubifs/bin/rfd | cut -d' ' -f3 | awk -v TEMP=$TEMP '
	{
		if ($N == TEMP "/ubifs/lib/ld-linux.so.3") {
			system("cp " TEMP "/ubifs/lib/ld-linux.so.3 " TEMP "/rfd/lib/");
		} else {
			system("cp " TEMP "/ubifs" $N " " TEMP "/rfd/lib/");
		}
	} '
	
	cp $TEMP/ubifs/bin/rfd $TEMP/rfd/bin/
	cp -R $TEMP/ubifs/firmware $TEMP/rfd/

	cat << EOF > $TEMP/rfd/etc/rfd_lan.conf

# TCP Port for XmlRpc connections
Listen Port = 2001


# Log Level: 1=DEBUG, 2=WARNING, 3=INFO, 4=NOTICE, 5=WARNING, 6=ERROR
Log Level = 2


# If set to 1 the AES keys are stored in a file. Highly recommended.
Persist Keys = 1


Device Files Dir = $INSTALL_DIR/etc/rfd/devices
Key File = $INSTALL_DIR/etc/rfd/keys
Address File = $INSTALL_DIR/etc/rfd/ids
PID File = $INSTALL_DIR/var/rfd.pid
UDS File = $INSTALL_DIR/var/socket_rfd
Fire NACK Error Events = true


# Path relative to QEMU_LD_PREFIX
Device Description Dir = /firmware/rftypes
Firmware Dir = /firmware
Replacemap File = /firmware/rftypes/replaceMap/rfReplaceMap.xml


#Logging
Log Destination = File
Log Filename = $INSTALL_DIR/var/log/rfd-%Y%m%d.log

[Interface 0]
Type = Lan Interface
Serial Number = <SerialNr>
# The encryption key is found on a sticker on the lan interface
Encryption Key = <Encryption Key>
Description = First Lan Interface
#IP Address = <IP Address>

EOF


        cat << EOF > $TEMP/rfd/etc/rfd_ccu.conf

# TCP Port for XmlRpc connections
Listen Port = 2001


# Log Level: 1=DEBUG, 2=WARNING, 3=INFO, 4=NOTICE, 5=WARNING, 6=ERROR
Log Level = 2


# If set to 1 the AES keys are stored in a file. Highly recommended.
Persist Keys = 1


Device Files Dir = $INSTALL_DIR/etc/rfd/devices
Key File = $INSTALL_DIR/etc/rfd/keys
Address File = $INSTALL_DIR/etc/rfd/ids
PID File = $INSTALL_DIR/var/rfd.pid
UDS File = $INSTALL_DIR/var/socket_rfd
Fire NACK Error Events = true


# Path relative to QEMU_LD_PREFIX
Device Description Dir = /firmware/rftypes
Firmware Dir = /firmware
Replacemap File = /firmware/rftypes/replaceMap/rfReplaceMap.xml


#Logging
Log Destination = File
Log Filename = $INSTALL_DIR/var/log/rfd-%Y%m%d.log

[Interface 0]
Type = CCU2
ComPortFile = /dev/ttyAPP0
AccessFile = /dev/null
ResetFile = /dev/ccu2-ic200

EOF




       cat << EOF > $TEMP/rfd/rfd-lan

#! /bin/sh
### BEGIN INIT INFO
# Provides: rfd-lan
# Required-Start: 
# Required-Stop: 
# Default-Start: 2 3 4 5
# Default-Stop: 0 1 6
# Short-Description: RFD Lan Interface Start/Stop
# Description: RFD Lan Adapter Start/Stop
# 
### END INIT INFO

RFD_DIR=$INSTALL_DIR

case "\$1" in
 start)
   su rfd -c "/usr/bin/qemu-arm -L \$RFD_DIR \$RFD_DIR/bin/rfd -f \$RFD_DIR/etc/rfd_lan.conf &"
   ;;
 stop)
   kill \`cat \$RFD_DIR/var/rfd.pid\`
   ;;
 restart)
   kill \`cat \$RFD_DIR/var/rfd.pid\`
   su rfd -c "/usr/bin/qemu-arm -L \$RFD_DIR \$RFD_DIR/bin/rfd -f \$RFD_DIR/etc/rfd_lan.conf &"
   ;;
 *)
   echo "Usage: rfd-lan {start|stop|restart}" >&2
   exit 3
   ;;
esac

EOF


       cat << EOF > $TEMP/rfd/rfd-ccu

#! /bin/sh
### BEGIN INIT INFO
# Provides: rfd-ccu
# Required-Start:
# Required-Stop:
# Default-Start: 2 3 4 5
# Default-Stop: 0 1 6
# Short-Description: RFD CCU Start/Stop
# Description: RFD CCU Start/Stop
#
### END INIT INFO

RFD_DIR=$INSTALL_DIR

case "\$1" in
 start)
   su rfd -c "/usr/bin/qemu-arm -L \$RFD_DIR \$RFD_DIR/bin/rfd -f \$RFD_DIR/etc/rfd_ccu.conf &"
   ;;
 stop)
   kill \`cat \$RFD_DIR/var/rfd.pid\`
   ;;
 restart)
   kill \`cat \$RFD_DIR/var/rfd.pid\`
   su rfd -c "/usr/bin/qemu-arm -L \$RFD_DIR \$RFD_DIR/bin/rfd -f \$RFD_DIR/etc/rfd_ccu.conf &"
   ;;
 *)
   echo "Usage: rfd-ccu {start|stop|restart}" >&2
   exit 3
   ;;
esac

EOF


}


fnc_install() {

	mkdir -p $INSTALL_DIR 2>/dev/null
	adduser --system --home $INSTALL_DIR --shell /bin/bash --no-create-home --group rfd >/dev/null
	cp -R $TEMP/rfd/* $INSTALL_DIR/
	chown -R rfd:rfd $INSTALL_DIR
}


fnc_prepare
fnc_download
fnc_mount_firmware
fnc_copy_files
fnc_install
